@@ -2,7 +2,7 @@ import python_stretch as m
 import numpy as np
 
 def test_mono():
-    x1 = np.random.rand(1, 44100)
+    x1 = np.random.normal(0, 0.1, size=(1, 44100)).astype(np.float32)
     
     ps = m.Signalsmith.Stretch()
     ps.setTransposeSemitones(12)
@@ -12,7 +12,7 @@ def test_mono():
     assert y1.shape == (1, 44100)
     
 def test_stereo():
-    x1 = np.random.rand(2, 44100)
+    x1 = np.random.normal(0, 0.1, size=(2, 44100)).astype(np.float32)
     
     ps = m.Signalsmith.Stretch()
     ps.setTransposeSemitones(12)
@@ -22,8 +22,8 @@ def test_stereo():
     assert y1.shape == (2, 44100)
     
 def test_multichannel():
-    nChannels = np.random.randint(3, 10)
-    x1 = np.random.rand(nChannels, 44100)
+    n_channels = np.random.randint(3, 10)
+    x1 = np.random.normal(0, 0.1, size=(n_channels, 44100)).astype(np.float32)
     
     ps = m.Signalsmith.Stretch()
     ps.setTransposeSemitones(12)
@@ -31,5 +31,5 @@ def test_multichannel():
     y1 = ps.process(x1)
     del ps
     
-    assert y1.shape == (nChannels, 44100)
+    assert y1.shape == (n_channels, 44100)
     
\ No newline at end of file
